@@ -27,7 +27,6 @@
 #include <stdlib.h>
 #include <stdarg.h>
 #include <stdio.h>
-#include <pwd.h>
 #include <string.h>
 #include <sys/types.h>
 #include <errno.h>
@@ -42,7 +41,6 @@
 #include "gnome-marshal.h"
 #include <gtk/gtk.h>
 #include <gdk/gdkprivate.h>
-#include <X11/Xatom.h>
 
 #include <libgnomeuiP.h>
 
@@ -2518,7 +2516,7 @@ gnome_client_save_any_dialog (GnomeClient *client, GtkDialog *dialog)
  * @dialog: Pointer to GNOME dialog widget (a #GtkDialog widget).
  *
  * Description:
- * May be called during a "save_youself" handler when an error has occured
+ * May be called during a "save_youself" handler when an error has occurred
  * during the save. The session manager decides when the dialog is shown, but
  * it will not be shown it unless the session manager is sending an interaction
  * style of #GNOME_INTERACT_ANY. A "Cancel Logout" button will be added
