@@ -30,8 +30,11 @@
  * Authors: Federico Mena <federico@nuclecu.unam.mx>
  */
 #include <config.h>
+#include <glib.h>
 #include <sys/param.h>
+#ifndef G_OS_WIN32
 #include <pwd.h>
+#endif
 #include <string.h>
 #include <unistd.h>
 #include <sys/types.h>
@@ -64,11 +67,13 @@ struct _GnomeFileEntryPrivate {
 
 	char *browse_dialog_title;
 
-	gboolean is_modal : 1;
+	GtkFileChooserAction filechooser_action;
 
-	gboolean directory_entry : 1; /*optional flag to only do directories*/
+	guint is_modal : 1;
 
-	gboolean use_filechooser : 1;
+	guint directory_entry : 1; /*optional flag to only do directories*/
+
+	guint use_filechooser : 1;
 
 	/* FIXME: Non local files!! */
 	/* FIXME: executable_entry as used in gnome_run */
@@ -101,6 +106,10 @@ static void fentry_get_property (GObject *object, guint param_id,
 
 static void gnome_file_entry_editable_init (GtkEditableClass *iface);
 
+#ifdef G_OS_WIN32
+#define realpath(a, b) strcpy (b, a)
+#endif
+
 /* Property IDs */
 enum {
 	PROP_0,
@@ -112,7 +121,8 @@ enum {
 	PROP_DEFAULT_PATH,
 	PROP_GNOME_ENTRY,
 	PROP_GTK_ENTRY,
-	PROP_USE_FILECHOOSER
+	PROP_USE_FILECHOOSER,
+	PROP_FILECHOOSER_ACTION
 };
 
 /* Note, can't use boilerplate with interfaces yet,
@@ -302,6 +312,16 @@ gnome_file_entry_class_init (GnomeFileEntryClass *class)
 						 FALSE,
 						 G_PARAM_READWRITE));
 
+	g_object_class_install_property (gobject_class,
+					 PROP_FILECHOOSER_ACTION,
+					 g_param_spec_enum (
+						 "filechooser_action",
+						 _("GtkFileChooser Action"),
+						 _("The type of operation that the file selector is performing."),
+						 GTK_TYPE_FILE_CHOOSER_ACTION,
+						 GTK_FILE_CHOOSER_ACTION_OPEN,
+						 G_PARAM_READWRITE));
+
 	class->browse_clicked = browse_clicked;
 	class->activate = NULL;
 }
@@ -346,6 +366,11 @@ fentry_set_property (GObject *object, guint param_id,
 	case PROP_USE_FILECHOOSER:
 		priv->use_filechooser = g_value_get_boolean (value);
 		break;
+
+	case PROP_FILECHOOSER_ACTION:
+		priv->filechooser_action = g_value_get_enum (value);
+		break;
+
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 		break;
@@ -404,6 +429,10 @@ fentry_get_property (GObject *object, guint param_id,
 		g_value_set_boolean (value, priv->use_filechooser);
 		break;
 
+	case PROP_FILECHOOSER_ACTION:
+		g_value_set_enum (value, priv->filechooser_action);
+		break;
+
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 		break;
@@ -430,6 +459,7 @@ browse_dialog_ok (GtkWidget *widget, gpointer data)
 
 	utf8_filename = g_filename_to_utf8 (locale_filename, -1, NULL,
 					    NULL, NULL);
+	g_free (locale_filename);
 	gtk_entry_set_text (GTK_ENTRY (entry), utf8_filename);
 	g_free (utf8_filename);
 	/* Is this evil? */
@@ -455,6 +485,7 @@ browse_dialog_kill (GtkWidget *widget, gpointer data)
 	fentry->fsw = NULL;
 }
 
+#ifndef G_OS_WIN32
 /* Does tilde (home directory) expansion on a string */
 static char *
 tilde_expand (const char *str)
@@ -493,6 +524,7 @@ tilde_expand (const char *str)
 
 	return g_strconcat (passwd->pw_dir, G_DIR_SEPARATOR_S, p, NULL);
 }
+#endif
 
 /* This is only used for the file dialog.  It can end up also being
  * the default path. */
@@ -510,11 +542,11 @@ build_filename (GnomeFileEntry *fentry)
 		(GTK_ENTRY (gnome_file_entry_gtk_entry (fentry)));
 
 	if (text == NULL || text[0] == '\0')
-		return g_strconcat (fentry->default_path, "/", NULL);
+		return g_strconcat (fentry->default_path, G_DIR_SEPARATOR_S, NULL);
 
 	file = _gnome_file_entry_expand_filename (text, fentry->default_path);
 	if (file == NULL)
-		return g_strconcat (fentry->default_path, "/", NULL);
+		return g_strconcat (fentry->default_path, G_DIR_SEPARATOR_S, NULL);
 
 	/* Now append a '/' if it doesn't exist and we're in directory only
 	 * mode.  We also have to do this if the file exists and it *is* a
@@ -526,7 +558,7 @@ build_filename (GnomeFileEntry *fentry)
 	len = strlen (file);
 
 	if (len != 0
-	    && file[len - 1] != G_DIR_SEPARATOR
+	    && !G_IS_DIR_SEPARATOR (file[len - 1])
 	    && (fentry->_priv->directory_entry || g_file_test (file, G_FILE_TEST_IS_DIR))) {
 		gchar *tmp;
 
@@ -596,16 +628,25 @@ browse_clicked(GnomeFileEntry *fentry)
 		if (fentry->_priv->directory_entry)
 			action = GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER;
 		else
-			action = GTK_FILE_CHOOSER_ACTION_OPEN;
+			action = fentry->_priv->filechooser_action;
 
 		fentry->fsw = gtk_file_chooser_dialog_new (fentry->_priv->browse_dialog_title
 							   ? fentry->_priv->browse_dialog_title
 							   : _("Select file"),
 							   NULL,
-							   GTK_FILE_CHOOSER_ACTION_OPEN,
+							   action,
 							   GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-							   GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
 							   NULL);
+
+		if (action == GTK_FILE_CHOOSER_ACTION_SAVE)
+			gtk_dialog_add_button (GTK_DIALOG (fentry->fsw),
+					       GTK_STOCK_SAVE,
+					       GTK_RESPONSE_ACCEPT);
+		else
+			gtk_dialog_add_button (GTK_DIALOG (fentry->fsw),
+					       GTK_STOCK_OPEN,
+					       GTK_RESPONSE_ACCEPT);
+
 		fw = fentry->fsw;
 
 		gtk_dialog_set_default_response (GTK_DIALOG (fw), GTK_RESPONSE_ACCEPT);
@@ -701,7 +742,7 @@ gnome_file_entry_drag_data_received (GtkWidget        *widget,
 
 	entry = gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (widget));
 
-	uris = g_strsplit (selection_data->data, "\r\n", -1);
+	uris = g_strsplit ((char *)selection_data->data, "\r\n", -1);
 	if (uris == NULL) {
 		gtk_drag_finish (context, FALSE, FALSE, time);
 		return;
@@ -966,17 +1007,21 @@ _gnome_file_entry_expand_filename (const char *input, const char *default_dir)
 		return NULL;
 	} else if (input[0] == '\0') {
 		return g_strdup ("");
-	} else if (input[0] == G_DIR_SEPARATOR) {
+	} else if (g_path_is_absolute (input)) {
 		return g_strdup (input);
+#ifndef G_OS_WIN32
 	} else if (input[0] == '~') {
 		return tilde_expand (input);
+#endif
 	} else if (default_dir != NULL) {
 		char *ret = g_build_filename (default_dir, input, NULL);
+#ifndef G_OS_WIN32
 		if (ret[0] == '~') {
 			char *tmp = tilde_expand (ret);
 			g_free (ret);
 			return tmp;
 		}
+#endif
 		return ret;
 	} else {
 		char *cwd = g_get_current_dir ();
