@@ -132,6 +132,10 @@ create_newwin(gboolean normal, gchar *appname, gchar *title)
 					   BONOBO_OBJREF(app->ui_container),
 					   NULL);
 
+	/* This is a test program only run on the build system, so
+	 * it's OK to use GNOMEUISRCDIR which is valid of course only
+	 * on the build system.
+	 */
 	bonobo_ui_util_set_ui (app->ui_component, "",
 			       GNOMEUISRCDIR "/testgnome.xml",
 			       appname, NULL);
@@ -141,6 +145,13 @@ create_newwin(gboolean normal, gchar *appname, gchar *title)
 	return app;
 }
 
+
+static void
+color_set (GnomeColorPicker *cp, guint r, guint g, guint b, guint a)
+{
+	g_print ("Color set: %d %d %d %d\n", r, g, b, a);
+}
+
 /* Creates a color picker with the specified parameters */
 static void
 create_cp (GtkWidget *table, int dither, int use_alpha, int left, int right, int top, int bottom)
@@ -148,6 +159,9 @@ create_cp (GtkWidget *table, int dither, int use_alpha, int left, int right, int
 	GtkWidget *cp;
 
 	cp = gnome_color_picker_new ();
+	g_signal_connect (cp, "color_set",
+			  G_CALLBACK (color_set), NULL);
+
 	gnome_color_picker_set_dither (GNOME_COLOR_PICKER (cp), dither);
 	gnome_color_picker_set_use_alpha (GNOME_COLOR_PICKER (cp), use_alpha);
 	gnome_color_picker_set_d (GNOME_COLOR_PICKER (cp), 1.0, 0.0, 1.0, 0.5);
@@ -382,7 +396,12 @@ create_druid(void)
 	GSList *radio_group;
 	druid_data *data;
 
-	/* load the images */
+	/* Load the images */
+	
+	/* This is a test program only run on the build system, so
+	 * it's OK to use the configuration-time GNOMEUIPIXMAPDIR from
+	 * the Makefile also on Win32.
+	 */
 	fname = g_strconcat (GNOMEUIPIXMAPDIR, "gnome-logo-icon.png", NULL);
 	if (fname)
 		/* FIXME: We must test GError */
@@ -651,18 +670,53 @@ create_file_entry(void)
 /*
  * IconEntry
  */
+static void
+icon_entry_changed (GnomeIconEntry *entry, GtkLabel *label)
+{
+	char *file = gnome_icon_entry_get_filename (entry);
+	g_print ("Entry changed, new icon: %s\n",
+		 file ? file : "Nothing selected");
+	g_free (file);
+}
+static void
+get_icon (GtkWidget *button, GnomeIconEntry *entry)
+{
+	GtkLabel *label = g_object_get_data (G_OBJECT (button), "label");
+	char *file = gnome_icon_entry_get_filename (entry);
+	gtk_label_set_text (label, file ? file : "Nothing selected");
+	g_free (file);
+}
+
 static void
 create_icon_entry(void)
 {
 	TestGnomeApp *app;
+	GtkWidget *vbox;
+	GtkWidget *button;
+	GtkWidget *label;
 	GtkWidget *entry;
 
 	app = create_newwin (TRUE, "testGNOME", "Icon Entry");
 
-	entry = gnome_icon_entry_new ("Foo", "Icon");
+	vbox = gtk_vbox_new (FALSE, 5);
 
-	bonobo_window_set_contents (BONOBO_WINDOW (app->app), entry);
-	gtk_widget_show (entry);
+	entry = gnome_icon_entry_new ("Foo", "Icon");
+	gtk_box_pack_start (GTK_BOX (vbox), entry, TRUE, TRUE, 0);
+
+	button = gtk_button_new_with_label ("Update label below");
+	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
+
+	label = gtk_label_new ("Nothing selected");
+	gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
+
+	g_signal_connect (entry, "changed",
+			  G_CALLBACK (icon_entry_changed), NULL);
+	g_signal_connect (button, "clicked",
+			  G_CALLBACK (get_icon), entry);
+	g_object_set_data (G_OBJECT (button), "label", label);
+
+	bonobo_window_set_contents (BONOBO_WINDOW (app->app), vbox);
+	gtk_widget_show_all (vbox);
 	gtk_widget_show (app->app);
 }
 
